use serde::{Serialize, Deserialize};

#[derive(Serialize, Deserialize)]
#[serde(rename_all = "snake_case")]
pub enum ClientMessage {
    CreateGame{ game_id: String, nick: String },
    Join{ game_id: String, nick: String },
    LeaveLobby,
    Ready,
    SubmitWord{ word: String },
    SubmitGuess{ guesses: Vec<(String, String)> },
}

#[derive(Serialize, Deserialize)]
#[serde(rename_all = "snake_case")]
pub enum GameStateData {
    Starting,
    Creating{ question: String, available_chars: Vec<char> },
    Guessing{ submitted_words: Vec<(String, Vec<char>)>, questions: Vec<String> },
    Revealing{  },
}

#[derive(Serialize, Deserialize)]
#[serde(rename_all = "snake_case")]
pub struct PlayerData {
    pub nick: String,
    pub points: i64
}

#[derive(Serialize, Deserialize)]
#[serde(rename_all = "snake_case")]
pub struct GameData {
    pub players: Vec<PlayerData>,
    pub state_data: GameStateData
}

#[derive(Serialize, Clone)]
#[serde(rename_all = "snake_case")]
pub struct RoundResultData {
    pub words: Vec<String>,
    pub questions: Vec<String>,
    pub solutions: Vec<(String, String)>,
    pub guesses: Vec<Vec<Vec<String>>>,
    pub point_list: Vec<(String, i64)>
}

#[derive(Serialize)]
#[serde(rename_all = "snake_case")]
pub enum UpdateMessage {
    GameNotFound{ game_id: String },
    GameAlreadyExists{ game_id: String },
    LeftLobby{ nick: String },
    GameState(GameData),
    RoundResult(RoundResultData),
}


